//package javazine.znetutil;

import java.net.*;
import java.io.*;
import java.util.*;
import javazine.zvector.*;

/*
 ZNewsReader 1.0   97/4/15
 Copyright (c) 1997, Andre Pinheiro (adlp@camoes.rnl.ist.utl.pt)
 All rights reserved.

 Published by JavaZine - Your Java webzine
 Links to JavaZine's websites
   - http://camoes.rnl.ist.utl.pt/~adlp/JavaZine/Links/JavaZine.html

 Permission to use, copy, modify, and distribute this software
 and its documentation for NON-COMMERCIAL or COMMERCIAL purposes
 and without fee is hereby granted provided that the copyright
 and "published by" messages above appear in all copies.
 We will not be held responsible for any unwanted effects due to
 the usage of this software or any derivative.
 No warrantees for usability for any specific application are
 given or implied.

 This application communicates with a news server, and:
   - lists all newsgroups;
   - displays all articles from selected newsgroups.
*/

public class ZNewsReader
{
  String host; // NNTP (Network News Transfer Protocol) server
  BufferedReader in;
  BufferedWriter out;
  String buffer;
  ZVectorString newsgroups = new ZVectorString();


  public ZNewsReader(String args[])
    throws IOException
  {
    this.host = args[0];

    Socket socket = null;

    socket = connect();
    try
    {
      open();
      handleParameters(args);
    }
    finally
    {
      socket.close();
    }
  }


  void handleParameters(String args[])
    throws IOException
  {
    if (args[1].equalsIgnoreCase("-l"))
    {
      listNewsgroups();
    }
    else
    if (args[1].equalsIgnoreCase("-d"))
    {
      for (int i = 2; i < args.length; i++)
        newsgroups.addElement(args[i]);

      if (newsgroups.size() < 1)
      {
        System.out.println("Error: no newsgroups specified.");
        System.exit(1);
      }

      displaySelectedNewsgroups();
    }
    else
    {
      System.out.println("Error: invalid parameters.");
      System.exit(1);
    }
  }


  Socket connect()
    throws IOException
  {
    int port = 119;

    System.out.println("Connecting to " + host + "...");
    Socket socket = new Socket(host, port);
    System.out.println("Connected.");

    out = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
    in = new BufferedReader(new InputStreamReader(socket.getInputStream()));

    return socket;
  }


  void open()
    throws IOException
  {
    buffer = in.readLine();
    if (buffer.charAt(0) != '2')
    {
      System.out.println("Error: news server not found.");
      System.exit(1);
    }
  }


  void sendCommand(String cmd)
    throws IOException, ZNewsCommandException
  {
    System.out.println("Sending command \"" + cmd + "\"...");
    out.write(cmd + "\r\n");
    out.flush();

    if ((buffer = in.readLine()) == null)
      throw new IOException();
    else
    {
      if (buffer.length() < 1)
        throw new IOException();

      if (buffer.charAt(0) != '2')
        throw new ZNewsCommandException("Command \"" + cmd + "\" failed.");
    }
  }


  String getTextResponse()
    throws IOException
  {
    String text = "";

    while(true)
    {
      buffer = in.readLine();

      if (buffer == null)
        throw new IOException("End of text not expected.");

      if (buffer.length() < 1)
        continue;

      buffer += "\n";

      // check for end of text
      if (buffer.charAt(0) == '.')
      {
        if (buffer.charAt(1) != '.')
          break; // no doubt, text ended

        // we've got a doubled period here
        // collapse the doubled period to a single one
        buffer = buffer.substring(1, buffer.length());
      }

      text += buffer;
    }

    return text;
  }


  void listNewsgroups()
    throws IOException
  {
    try
    {
      sendCommand("list");
    }
    catch(Throwable e)
    {
      System.out.println(e.getMessage());
      return;
    }

    // get names of all newsgroups
    while(true)
    {
      buffer = in.readLine();

      if (buffer == null)
        throw new IOException("End of text not expected.");

      if (buffer.length() < 1)
        continue;

      if (buffer.charAt(0) == '.') // check for end of listing
        break;

      StringTokenizer st = new StringTokenizer(buffer, " ");
      newsgroups.addElement(st.nextToken());
    }

    for(int i = 0; i < newsgroups.size(); i++)
      System.out.println(newsgroups.elementAt(i).toString());
  }


  void displaySelectedNewsgroups()
    throws IOException
  {
    for(int i = 0; i < newsgroups.size(); i++)
      displayNewsgroup(newsgroups.elementAt(i).toString());
  }


  void displayNewsgroup(String group)
    throws IOException
  {
    try
    {
      sendCommand("group " + group);
    }
    catch(ZNewsCommandException e)
    {
      System.out.println("Error: could not access newsgroup \"" + group +"\".");
      return;
    }
    catch(IOException e)
    {
      System.out.println(e.getMessage());
      return;
    }

    StringTokenizer st = new StringTokenizer(buffer, " ");
    st.nextToken();
    st.nextToken();
    int firstArticleNumber = (new Integer(st.nextToken())).intValue(),
        lastArticleNumber = (new Integer(st.nextToken())).intValue();
    System.out.println("First article #" + firstArticleNumber);
    System.out.println("Last article #" + lastArticleNumber);

    while(true)
    {
      try
      {
        sendCommand("head");
        System.out.println(getTextResponse()); // display header text
        sendCommand("body");
        System.out.println(getTextResponse()); // display body text
      }
      catch(Throwable e)
      {
        System.out.println(e.getMessage());
        // there's been a problem with the current article, let's try to get the next one
      }

      try
      {
        sendCommand("next");
      }
      catch(ZNewsCommandException e)
      {
        // "next" command failed
        // looks like there are no more articles in the current newsgroup
        System.out.println(e.getMessage());
        return;
      }
      catch(IOException e)
      {
        System.out.println(e.getMessage());
        // an IO error, let's get out of here!
        return;
      }
    }
  }


  public static void about()
  {
    System.out.println("\n ZNewsReader <news server> -command [<newsgroup1>] [<newsgroup...>]\n"
      + "\nCommands:\n"
      + "  l  List all newsgroups.\n"
      + "  d  Display all articles from specified newsgroups.");
    System.exit(1);
  }


  public static void main(String args[]) throws IOException
  {
    if (args.length == 1)
      if (args[0].equals("-?"))
        about();

    if (args.length < 2)
    {
      System.out.println("Error: invalid parameters.");
      System.exit(1);
    }

    new ZNewsReader(args);
  }
}


class ZNewsCommandException extends Exception
{
  /**
   * Constructs a <code>ZNewsCommandException</code> with no detail message.
   */
  public ZNewsCommandException()
  {
    super();
  }

  /**
   * Constructs a <code>ZNewsCommandException</code> with the specified detail message.
   *
   * @param s  The detail message.
   */
  public ZNewsCommandException(String s)
  {
    super(s);
  }
}
